#
# digitize.rb
#   ruby script to digitize a displayed graph image.
#   This uses many libraries (packages), such as libtcltk-ruby and libtk-img.
#
#------------------------------------------------------------------------------
# 2013/10/27, yot
# 2015/12/21, Add GUI file section, yot
#------------------------------------------------------------------------------
# Usage:
#  * Run this script
#      $ ruby digitize.rb
#  * Select an input png file
#  * Select/Input an output file
#  * Determine axes:
#    * X direction
#      * Click point 1 and enter x coordinate of the point
#        * The y coordinate of the point 1 is not necessarily zero.
#      * Click point 2 and enter x coordinate of the point
#        * The y coordinate of the point 2 is the same as that of point 1.
#    * Y direction
#      * Click point 3 and enter y coordinate of the point
#        * The x coordinate of the point 3 is not necessarily zero.
#      * Click point 4 and enter y coordinate of the point
#        * The x coordinate of the point 4 is the same as that of point 3.
#  * Trace a line by using a mouse left button or press a mouse right button 
#    to exit
#  * To finish the script, press a mouse right button
#------------------------------------------------------------------------------
#
# Reference for ruby tk
#   http://ronor.web.fc2.com/rubytk/
#   http://pub.cozmixng.org/~the-rwiki/?cmd=view;name=%B5%D5%B0%FA%A4%ADRuby%2FTk
#   http://www.mnet.ne.jp/~tnomura/tksample.html
#   http://www.dumbo.ai.kyutech.ac.jp/nomura-ken/kajiyama/ruby_tk/getfile.html
#
#!/usr/bin/env ruby

require "tk"
require "tkextlib/tkimg/png"
include Math


class Click
  def initialize( outfn )
    @count = 0
    print "Determine x direction\n"
    print "  Click a point to determine x direction\n"
    @file = File.open(outfn,'w')
  end
  def closefile
    @file.close
  end
  def clickevent( x, y )
    @count += 1
#    print @count, "\n"
#    print "    (", x, ',', y, ")\n"
    if @count == 1 then
      print "  Enter x coordinate of the clicked point: "
      @p1_x_coord = gets.to_f
      @p1_x_px    = x
      @p1_y_px    = y
      print "  Click another point which has the same y with the previous point\n"
    elsif @count == 2 then
      print "  Enter x coordinate of the clicked point: "
      @p2_x_coord = gets.to_f
      @p2_x_px    = x
      @p2_y_px    = y
      print "Determine y direction\n"
      print "  Click a point to determine y direction\n"
    elsif @count == 3 then
      print "  Enter y coordinate of the clicked point: "
      @p3_y_coord = gets.to_f
      @p3_x_px    = x
      @p3_y_px    = y
      print "  Click another point which has the same x with the previous point\n"
    elsif @count == 4 then
      print "  Enter y coordinate of the clicked point: "
      @p4_y_coord = gets.to_f
      @p4_x_px    = x
      @p4_y_px    = y
      print "  Clicked coordinate:\n"
      print "    x1 : ", @p1_x_coord, ", x2: ", @p2_x_coord, "\n"
      print "    y1 : ", @p3_y_coord, ", y2: ", @p4_y_coord, "\n"
      print "Trace line/point\n"
      print "  Click a point by left button or press right button to exit\n"
      # Determine coordinate (in pixel) of the origin
      #   pixel of x = 0 on "x axis"
      a = - @p1_x_coord / ( @p2_x_coord - @p1_x_coord )
      x0_on_x_px = @p1_x_px + a * ( @p2_x_px - @p1_x_px )
      y0_on_x_px = @p1_y_px + a * ( @p2_y_px - @p1_y_px )
      #   pixel of y = 0 on "x axis"
      b = - @p3_y_coord / ( @p4_y_coord - @p3_y_coord )
      x0_on_y_px = @p3_x_px + b * ( @p4_x_px - @p3_x_px )
      y0_on_y_px = @p3_y_px + b * ( @p4_y_px - @p3_y_px )
      #
      d = -   (   ( x0_on_x_px - x0_on_y_px ) * ( @p2_y_px - @p1_y_px )   \
                - ( y0_on_x_px - y0_on_y_px ) * ( @p2_x_px - @p1_x_px ) ) \
            / (   ( @p4_x_px   - @p3_x_px   ) * ( @p2_y_px - @p1_y_px )   \
                - ( @p4_y_px   - @p3_y_px   ) * ( @p2_x_px - @p1_x_px ) )
      c =       ( x0_on_x_px - x0_on_y_px ) / ( @p2_x_px - @p1_x_px ) \
          + d * ( @p4_x_px   - @p3_x_px   ) / ( @p2_x_px - @p1_x_px )
      @or_x_px = x0_on_y_px + c * ( @p2_x_px - @p1_x_px )
      @or_y_px = y0_on_y_px + c * ( @p2_y_px - @p1_y_px )
#      print "ORIGIN (in pixel) : ", @or_x_px, " ", @or_y_px, "\n"
      # Determine unit vector for x direction (unit in dimensional unit)
      @x_unitvec_x_coord = ( @p2_x_px - @p1_x_px ) / ( @p2_x_coord - @p1_x_coord )
      @x_unitvec_y_coord = ( @p2_y_px - @p1_y_px ) / ( @p2_x_coord - @p1_x_coord )
      # Determine unit vector for y direction (unit in dimensional unit)
      @y_unitvec_x_coord = ( @p4_x_px - @p3_x_px ) / ( @p4_y_coord - @p3_y_coord )
      @y_unitvec_y_coord = ( @p4_y_px - @p3_y_px ) / ( @p4_y_coord - @p3_y_coord )
    else
      # Determine coordinate (in pixel) of a point on a y axis
      beta                                              \
        = (    ( x - @or_x_px ) * @x_unitvec_y_coord    \
             - ( y - @or_y_px ) * @x_unitvec_x_coord )  \
        / (   @y_unitvec_x_coord * @x_unitvec_y_coord   \
            - @x_unitvec_x_coord * @y_unitvec_y_coord )
      alpha = ( x - @or_x_px - beta * @y_unitvec_x_coord ) / @x_unitvec_x_coord
      x_coord = alpha
      y_coord = beta
      print x_coord, ", ", y_coord, "\n"
      @file.puts x_coord.to_s + " " + y_coord.to_s
    end
  end
end

# input file name
pngfn = Tk.getOpenFile( 'title'=>'Select an input file', 'filetypes'=>"{PNG {.png}}" )
# output file name
outfn = Tk.getSaveFile( 'title'=>'Select/Input an output file name' )

window = TkRoot.new {
  title pngfn
  resizable [0,0]
}

clickclass = Click.new( outfn )

TkButton.new {
  relief 'sunken'
  cursor 'plus'
  image TkPhotoImage.new("file" => pngfn)
  bind 'Button', 
    proc { |x,y| clickclass.clickevent( x, y ) }, "%x %y"
  bind 'Button-3', proc { TkRoot.new.destroy }
  pack
}

Tk.mainloop

clickclass.closefile()
print "Stop\n"
